/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.Logout");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "LogoutMessages");

/**
 * A simple Logout widget
 */
(function() {
var checkNotNull                 = net.jazz.ajax.toolbox.lang.checkNotNull;
var getAuthenticatedContributor  = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var getExpirationInfo            = com.ibm.team.repository.web.client.internal.licensing.getExpirationInfo;
var getLicenseInfo               = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;
var SRH                          = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FORM_SESSION_COOKIE			= "JazzFormAuth";
var X_AUTH_MSG                  = "X-com-ibm-team-repository-web-auth-msg";
var DEFAULT_LOGOUT_URI			= dojo.moduleUrl("com.ibm.team.repository.web", "loggedout.html");

function getAuthMethod() {
	return document.cookie.match(FORM_SESSION_COOKIE + "=") ? "FORM" : "BASIC";
}

function getBasicLogoutLinkUri() {
	return com.ibm.team.repository.web.logouturi;
}

function getFormLogoutLinkUri() {
	var uri = com.ibm.team.repository.web.logouturi;
	if (uri == null) {
		uri = DEFAULT_LOGOUT_URI;
	}
	uri += "?path=" + escape(this.location.pathname + this.location.search + this.location.hash);
	return uri;
}

dojo.declare("com.ibm.team.repository.web.ui.internal.Logout", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/Logout.html"),
	//templateCssPath: dojo.uri.dojoUri("../com.ibm.team.repository.web/ui/internal/templates/Logout.css"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "LogoutMessages");
	},
	
	/**
	 * Fill in the dynamic elements of the header nav.
	 */
	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.Logout.superclass.postCreate.apply(this);
		checkNotNull(this.userIdContainer, "this.useridContainer");
		checkNotNull(this.logoutLinkContainer, "this.logoutLinkContainer");
		
		if (this.loggedInUser == null) {
			this.loggedInUser = getAuthenticatedContributor().name;
		}
		this.userIdContainer.appendChild(document.createTextNode(this.loggedInUser));
		
		if (getAuthMethod() == "FORM") {
			this._addFormLogoutLink();
		}
		else if (getBasicLogoutLinkUri()) {
			this._addBasicLogoutLink();
		}
		
		dojo.subscribe(com.ibm.team.repository.web.internal.events.licenseChange, this, "_showLicenseInfo");
		var self = this;
		setTimeout(function() {
			getLicenseInfo(new SRH(self, "_showLicenseInfo", "_showLicenseInfoErr"));
		}, 4000);
	},
	
	_showLicenseInfo: function(licenseInfo) {
		var disabledReason = dojo.getObject("serverLicense.disabledReason", false, licenseInfo || "");
		var expirationTime = dojo.getObject("serverLicense.expirationTime", false, licenseInfo || "");
		
		// if there's no expiration time, don't show anything
		if (!expirationTime && !disabledReason) {
			this.expirationInfo.innerHTML = "";
			this.expirationInfoSeparator.style.display = "none";
		} else if(disabledReason) {
			this.expirationInfo.className = "expirationWarning";
			this.expirationInfo.innerHTML = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>" + disabledReason + "</a> ";
		} else {
			var expirationInfo = getExpirationInfo(expirationTime);
			var purchaseLink = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>";
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.trialLicense, [ purchaseLink, "</a> "]) + " ";
			if(expirationInfo.isExpired || expirationInfo.daysRemaining <= 14) {
				this.expirationInfo.className = "expirationWarning";
			} else {
				this.expirationInfo.className = "expirationInfo";
			}
			if(expirationInfo.isExpired) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [ this.expirationInfo.innerHTML ]) + " ";
			} else if (expirationInfo.daysRemaining === 0) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [ this.expirationInfo.innerHTML ]) + " ";
			} else if (expirationInfo.daysRemaining === 1) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [ this.expirationInfo.innerHTML ]) + " ";
			} else {
				var daysLeft = expirationInfo.daysRemaining;
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [ this.expirationInfo.innerHTML, daysLeft ]) + " ";
			}
			this.expirationInfoSeparator.style.display = "inline";
		}
	},
	
	_showLicenseInfoErr: function(errorObj) {
		console.warn(dojo.string.substitute(this.messages.noLicenseInfo, [ errorObj.message ]));
	},
	
	_addBasicLogoutLink: function() {
		checkNotNull(getBasicLogoutLinkUri(), "getBasicLogoutLinkUri()");
		
		this.logoutLinkContainer.appendChild(document.createTextNode(" | "));
		
		var logoutLink = document.createElement("a");
		logoutLink.href = getBasicLogoutLinkUri(true);
		
		logoutLink.appendChild(document.createTextNode("Log Out"));
		
		this.logoutLinkContainer.appendChild(logoutLink);
	},
	
	_addFormLogoutLink: function () {
		this.logoutLinkContainer.appendChild(document.createTextNode(" | "));
		
		var logoutLink = document.createElement("a");
		var logoutURI = getFormLogoutLinkUri();
		logoutLink.href = "javascript:{}";
		logoutLink.onclick = function () {
			var bindArgs = {
				url: net.jazz.ajax._contextRoot + "/auth/logout",
				handleAs: "json",
				headers: {
					accept: "text/json"
				},
				preventCache: dojo.isIE,
				handle: function(response, ioArgs) {
					document.location.reload();
				}
			};
			dojo.xhrPost(bindArgs);
		}
		
		logoutLink.appendChild(document.createTextNode(this.messages.logout));
		this.logoutLinkContainer.appendChild(logoutLink);
	}
	
});

})();
