/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage");

dojo.require("dijit._Widget");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.SideNavigationLayout");

dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ActiveServicesView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ComponentsView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureAdvanced");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureDatabaseConnection");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureFeedSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureMailSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseUsageView");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseEditor");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ReloadLogSettings");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.Statistics");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ProvisionStatus");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerReset");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerPageMessages");


var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
var getLicenseInfo = LIC_P.getLicenseInfo;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerPage", dijit._Widget, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerPageMessages");
	},
	
	//
	// Methods intended to be called by child pages
	//
	
	reloadCurrent: function() {
		this._previousActionEntry = this._currentActionEntry || null;
		this._showCurrentWidget(/*isReload*/true);
	},	
	
	//
	// Dojo lifecycle and internal methods
	//
	
	postCreate: function() {
		// call superclass's postCreate method
		this.inherited(arguments);
		
		// initialize widget cache
		this._widgetCache = {};
		
		// set the root style for the Server page and all of its sub-pages
		this.domNode.className = "com-ibm-team-repository-admin";
		
		// instantiate the side-nav layout UI and append it to the page
		this._appView = new net.jazz.ajax.ui.SideNavigationLayout();
		this.domNode.appendChild(this._appView.domNode);
		
		// register actions and create side-nav hyperlinks
		ActionRegistry = getWorkbench().getActionRegistry();
		ActionRegistry.registerAction(PREFIX + "_defaultAction", this, "_defaultAction");
		
		var self = this;
		this.allLinks = [];
		dojo.forEach(ACTION_SETS, function(actionSet) {
			var navSection = document.createElement("div");
			self.connect(navSection, "onclick", self._onNavSectionClick);
			navSection.className = "com-ibm-team-repository-admin-navSection";
			if((actionSet.labelKey !== FLOATING_NAV_KEY && actionSet.labelKey !== INTERNAL_TOOLS_NAV_KEY)  ||  
			   (actionSet.labelKey === INTERNAL_TOOLS_NAV_KEY && location.search.indexOf("internal") !== -1)) {
				self._appView.addNavigationSection(navSection, self.messages[actionSet.labelKey], true);
			}  
			if (actionSet.labelKey == FLOATING_NAV_KEY){
				self.floatingNav = {
					section: navSection, 
					label: self.messages[actionSet.labelKey]
				};
			}
			dojo.forEach(actionSet.actions, function(action){
				var id = PREFIX + action.shortId;
				ACTION_MAP[action.shortId] = action;
				var link = getNavigationLink(id, self.messages[action.labelKey]);
				link.__jazzActionId = action.shortId;
				navSection.appendChild(link);
				var span = document.createElement("span");
				span.__jazzReload = true;
				link.appendChild(span);
				self.allLinks.push(link);
				ActionRegistry.registerAction(id, self, "_handleAction");
			});
		});
		
		this._fetchLicenseInfo();		
	},
	
	_defaultAction: function() {
		// by default, show the first section from the left-nav
		this._handleAction({action: PREFIX + ACTION_SETS[0].actions[0].shortId});
	},
	
	_handleAction: function(args) {
		if (!isAdmin()) {
			this._showUnauthorized();
			return;
		}

		if(!args.action) {
			console.debug("ServerPage.js#_handleAction: missing required param 'args.action'");
			return;
		}
		
		// cache the previous entry to see if the widget should be destroyed or
		// not
		this._previousActionEntry = this._currentActionEntry || null;
		
		// parse out the short id from the fully-qualified id
		var shortId = args.action.substring(args.action.lastIndexOf(".") + 1);
		this._currentActionEntry = ACTION_MAP[shortId];
		this._currentArgs = args;
		this._currentArgs.parent = this;
				
		// set the title for the action
		getWorkbench().setTitle(this.messages[this._currentActionEntry.labelKey]);
		
		// highlight the selected link and unhighlight the others
		dojo.forEach(this.allLinks, function(link){
			link.className = link.__jazzActionId === shortId ? "selected" : "";
		});
		
		// call the actual method implementation
		this._showCurrentWidget();
	},
	
	_onNavSectionClick: function(e) {
		if(e.target.__jazzReload) {
			dojo.stopEvent(e);
			this.reloadCurrent();
		}
	},
	
	_showCurrentWidget: function(isReload) {
		// save a reference to the prior content widget
		this._previousContentWidget = this._currentContentWidget || null;
		
		if(this._currentActionEntry.cached) {
			this._showCachedWidget(isReload);
		} else {
			this._showUncachedWidget();
		}
		
		// help the garbage collector do its job
		if(this._previousContentWidget && !this._previousActionEntry.cached) {
			this._previousContentWidget.destroyRecursive();
		}
	},
	
	_showCachedWidget: function(isReload) {
		var className = this._currentActionEntry.widget;
		if(this._widgetCache[className]) {
			// get widget from the cache if it exists
			this._currentContentWidget = this._widgetCache[className];
		} else {
			// otherwise instantiate it and add it to the cache
			var constructor = dojo.getObject(SERVER_PACKAGE + className);
			this._currentContentWidget = new constructor(this._currentArgs);
			dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
			this._widgetCache[className] = this._currentContentWidget;
		}
		if(isReload) {
			this._currentContentWidget.reload(this._currentArgs);
		} else {
			this._currentContentWidget.show(this._currentArgs);
			this._appView.setMainContent(this._currentContentWidget.domNode);
		}
	},
	
	_showUncachedWidget: function() {
		var constructor = dojo.getObject(SERVER_PACKAGE + this._currentActionEntry.widget);
		this._currentContentWidget = new constructor(this._currentArgs);
		dojo.addClass(this._currentContentWidget.domNode, "com-ibm-team-repository-admin-contentArea");
		this._appView.setMainContent(this._currentContentWidget.domNode);
	},
	
	_showUnauthorized: function(){
		dojox.data.dom.removeChildren(this.domNode);
		if (!this._notAuth) {
			this._notAuth = new com.ibm.team.repository.web.ui.internal.admin.NotAuthorizedView();
		}	
		this.domNode.appendChild(this._notAuth.domNode);
	}, 
	
	_fetchLicenseInfo: function () {
		var srh= new ServiceResponseHandler(this, "processLicenseInfoResponse", "handleLicenseErrorReponse");
		getLicenseInfo(srh);		
	},
	
	processLicenseInfoResponse: function(licenseInfo) {
		var isFloating = dojo.getObject("floatingSetupStatus.server", false, licenseInfo);
		if (isFloating) {
			this._showFloatingLicenseNavigation(isFloating);
		}
		dojo.subscribe(com.ibm.team.repository.web.internal.events.floatingLicenseChange, this, "receiveFloatingLicenseNotification");
	},
	
	
	handleLicenseErrorReponse: function(error) {
		console.info("Unable to determine if the server is a Floating License Server.")
		dojo.subscribe(com.ibm.team.repository.web.internal.events.floatingLicenseChange, this, "receiveFloatingLicenseNotification");
	},
	
	receiveFloatingLicenseNotification: function (licenseInfo) {
		if (licenseInfo.actingAsFloating) {
			this._showFloatingLicenseNavigation(licenseInfo.actingAsFloating);
		}
	},
	
	_showFloatingLicenseNavigation: function (isFloatingServer) {
		if (isFloatingServer && !this.floatingNavInitialized) {
			this._appView.addNavigationSection(this.floatingNav.section, this.floatingNav.label, true);
			this.floatingNavInitialized = true;
		}
	}
	
});

//
// aliases, constants, and module-private variables
//

var NoAuth         = com.ibm.team.repository.web.ui.internal.admin.NoAuth;
var PlatformUI     = net.jazz.ajax.ui.PlatformUI;
var ViewCounters   = com.ibm.team.repository.web.ui.internal.admin.server.ViewCounters;

var ActionRegistry = null;
var PREFIX         = "com.ibm.team.repository.admin.";
var SERVER_PACKAGE = "com.ibm.team.repository.web.ui.internal.admin.server.";
var ACTION_MAP     = {};

//
// The following defines the left-nav links and their bindings to actions and
// widgets.
//

var FLOATING_NAV_KEY = 'floatingLicense';
var INTERNAL_TOOLS_NAV_KEY = 'internalTools';
var ACTION_SETS = [
	{ labelKey: 'status', actions: [
		{labelKey: 'statusSummary', shortId: "serverStatus", widget: "ServerStatus"},
		{labelKey: 'statistics', shortId: "statistics", widget: "Statistics"},
		{labelKey: 'componentStatus', shortId: "componentStatus", widget: "ComponentsView"},
		{labelKey: 'activeServices', shortId: "activeServices", widget: "ActiveServicesView"}
	]},
	{ labelKey: 'configuration', actions: [
		{labelKey: 'emailSettings', shortId: "configureMailSettings", widget: "ConfigureMailSettings"},
		{labelKey: 'databaseConnection', shortId: "configureDatabaseConnection", widget: "ConfigureDatabaseConnection"},
		{labelKey: 'feedSettings', shortId: "configureFeedSettings", widget: "ConfigureFeedSettings"},
		{labelKey: 'licenseKeyManagement', shortId: "manageLicenses", widget: "LicenseEditor", cached: true},
		{labelKey: 'advancedProperties', shortId: "configureAdvanced", widget: "ConfigureAdvanced"}
	]},
	{ labelKey: FLOATING_NAV_KEY, actions: [
		{labelKey: 'reports', shortId: "floatingLicenseReports", widget: "FloatingLicenseReportView"},
		{labelKey: 'issuedLeases', shortId: "floatingLicenseUsage", widget: "FloatingLicenseUsageView"}
	]},	
	{ labelKey: INTERNAL_TOOLS_NAV_KEY, actions: [
		{labelKey: 'counters', shortId: "viewCounters", widget: "ViewCounters", cached: true},
		{labelKey: 'reloadLogSettings', shortId: "reloadLoggingSettings", widget: "ReloadLogSettings"},
		{labelKey: 'provisionStatus', shortId: "provisionStatus", widget: "ProvisionStatus"},
		{labelKey: 'serverReset', shortId: "serverReset", widget: "ServerReset"}
	]}
];

function getNavigationLink(action, title) {
	var a = document.createElement("a");
	a.href = "#action=" + action;
	a.appendChild(document.createTextNode(title));
	
	return a;
}

function getWorkbench() {
	return net.jazz.ajax.ui.PlatformUI.getWorkbench();
}

function isAdmin() {
	return com.ibm.team.repository.web.noAdminAuth ? false : true;
}

})();