/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers");

dojo.require("dojox.data.dom");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.ItemList");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageArchivedUsersMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

function setText(e, text){
	e.appendChild(document.createTextNode(text));
}

var ACTION_PREFIX = "com.ibm.team.repository.editArchivedUser&itemId=";
var SEARCH_ACTION = "action=com.ibm.team.repository.searchArchivedUsers";
var PAGE_SIZE = 50;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.ManageArchivedUsers", com.ibm.team.repository.web.ui.internal.admin.ItemList, {
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.users", "ManageArchivedUsersMessages");
	},
	
	postCreate: function() {
		setText(this.heading, this.messages.archivedUsers);
		setText(this.description, this.messages.pageDescription);
		this.tableHeading.parentNode.removeChild(this.tableHeading);
		this.newItemButtonArea.parentNode.removeChild(this.newItemButtonArea);
		this.tableControls.style.paddingTop = "0.5em";
		this.searchText = document.createElement("input");
		this.searchText.type = "text";
		this.tableControls.appendChild(this.searchText);
		this.connect(this.searchText,"onkeypress",function (e) {
			if (e.keyCode == 13) {
				document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.searchText.value);
			}
		});
		this.searchButton = document.createElement("button");
		this.searchButton.style.marginLeft = "5px";
		this.searchButton.appendChild(document.createTextNode(this.messages.search));
		this.tableControls.appendChild(this.searchButton);
		this.connect(this.searchButton,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.searchText.value);
		});
		
		this.setHeaderRow([this.messages.name, this.messages.userId, this.messages.email]);
		this.headerRow.cells[0].sortBy = "name";
		this.headerRow.cells[1].sortBy = "userId";
		this.headerRow.cells[2].sortBy = "emailaddress";
		dojo.toggleClass(this.headerRow, "sortable", true);
		this.connect(this.headerRow, "onclick", "updateSorting");
		this.connect(this.headerRow, "onmouseover", function (e) {
			dojo.toggleClass(e.target, "selected", true);
		});
		this.connect(this.headerRow, "onmouseout", function (e) {
			dojo.toggleClass(e.target, "selected", false);
		});
		
		this.pageNumberContainer = document.createElement("div");
		this.pageNumberContainer.style.marginTop = "5px";
		this.pageNumberContainer.style.fontWeight = "normal";
		this.pageNumberContainer.style.display = "none";
		this.pageNumberContainer.appendChild(document.createTextNode(dojo.string.substitute(this.messages.showingFirstUsers, [ PAGE_SIZE ])));
		this.tableControls.appendChild(this.pageNumberContainer);
		
		this.refresh();
	},
	
	refresh: function () {
		this.clearState();
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode(this.messages.searchingForUsers));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		if (this.latestSearch != null) {
			this.searchText.value = this.latestSearch.searchTerm;
			this.runUserSearch(this.latestSearch.searchTerm, this.latestSearch.pageNum);
		}
		else {
			this.runUserSearch("");
		}
	},
	
	show: function () {
		this.refresh();
	},
	
	reload: function () {
		this.refresh();
	},
	
	clearState: function () {
		this.searchText.value = "";
		dojox.data.dom.removeChildren(this.tbody);
		this.pageNumberContainer.style.display = "none";
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = "";
	},
	
	runUserSearch: function (searchTerm, pageNum, sortBy, sortOrder) {
		if (searchTerm == null) {
			searchTerm = "";
		}
		if (sortBy == null) {
			sortBy = "name";
		}
		if (sortOrder !== "desc") {
			sortOrder = "asc";
		}
		if (pageNum == null) {
			pageNum = 0;
		}
		this.latestSearch = {
			searchTerm: searchTerm,
			pageNum: pageNum,
			sortBy: sortBy,
			sortOrder: sortOrder
		};
		searchTerm = searchTerm.replace(/\*/g,"%");
		this.latestSearchTerm = searchTerm;
		this.latestSearchType = "searchTerm";
		this.queryPage = pageNum || 0;
		this.updatePagerDivLinks();
		dojox.data.dom.removeChildren(this.tbody);
		var tr = document.createElement("tr");
		var td = document.createElement("td");
		td.appendChild(document.createTextNode(this.messages.searchingForUsers));
		tr.appendChild(td);
		this.tbody.appendChild(tr);
		var srh = new ServiceResponseHandler(this, "showUsers", "handleError");
		AdminClient.getContributors({searchTerm: searchTerm, pageSize: PAGE_SIZE, sortBy: sortBy, sortOrder: sortOrder, hideGuest: true, hideUnassigned: true, pageNum: pageNum, onlyArchivedUsers: true}, srh);
	},
	
	updateSorting: function (e) {
		if (this.latestSearch) {
			var sortByField = e.target.sortBy;
			var sortOrder;
			if (this.latestSearch.sortBy === sortByField) {
				if (this.latestSearch.sortOrder === "desc") {
					sortOrder = "asc";
				}
				else {
					sortOrder = "desc";
				}
			}
			else {
				sortOrder = "asc";
			}
			document.location.href = "#" + SEARCH_ACTION + "&searchTerm=" + encodeURIComponent(this.latestSearch.searchTerm) + "&pageNum=" +
					this.latestSearch.pageNum + "&sortBy=" + sortByField + "&sortOrder=" + sortOrder;
		}
	},
	
	showUsers: function (resultSet) {
		dojox.data.dom.removeChildren(this.tbody);
		if (this.pagerDiv == null) {
			this.createPagerDiv();
		}
		if (resultSet == null || resultSet.elements == null || resultSet.elements.length == 0) {
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.appendChild(document.createTextNode(this.messages.noMatchingUsers));
			td.colSpan = 3;
			tr.appendChild(td);
			tr.onclick = function (e) {
				dojo.stopEvent(e);
			}
			this.tbody.appendChild(tr);
			dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
			dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			this.pagerDiv.rangeSpan.appendChild(document.createTextNode("0"));
			this.pagerDiv.totalSpan.appendChild(document.createTextNode("0"));
		}
		else {
			dojox.data.dom.removeChildren(this.pagerDiv.rangeSpan);
			dojox.data.dom.removeChildren(this.pagerDiv.totalSpan);
			if (resultSet.count > PAGE_SIZE) {
				var lowerBound = (this.queryPage * PAGE_SIZE) + 1;
				var upperBound = (this.queryPage + 1) * PAGE_SIZE;
				if (this.queryPage == 0) {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", false);
				}
				if (upperBound >= resultSet.count) {
					upperBound = resultSet.count;
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				}
				else {
					dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", false);
				}
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode(lowerBound + " - " + upperBound));
			}
			else {
				dojo.toggleClass(this.pagerDiv.prevSpan,"disabled", true);
				dojo.toggleClass(this.pagerDiv.nextSpan,"disabled", true);
				this.pagerDiv.rangeSpan.appendChild(document.createTextNode("1 - " + resultSet.count));
			}
			this.pagerDiv.totalSpan.appendChild(document.createTextNode(resultSet.count));
			dojo.forEach(resultSet.elements, dojo.hitch(this, function(user){
				var row = this.addRow(ACTION_PREFIX + user.itemId, [user.name, user.userId, user.emailAddress]);
				var archiveIcon = document.createElement("img");
				archiveIcon.id = "icon_" + user.itemId;
				archiveIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore.gif");
				archiveIcon.title = this.messages.unarchiveUser;
				archiveIcon.className = "archiveIcon";
				archiveIcon.userItemId = user.itemId;
				this.connect(archiveIcon,"onmouseover", function () {
					archiveIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore-hov.gif");
				});
				this.connect(archiveIcon, "onmouseout", function () {
					archiveIcon.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/restore.gif");
				});
				this.connect(archiveIcon, "onclick", function (e) {
					dojo.stopEvent(e);
					this.unarchiveUser(e.target.userItemId);
				});
				row.cells[0].insertBefore(archiveIcon, row.cells[0].firstChild);
				dojo.toggleClass(row.parentNode,"userTable",true);
			}));
		}
	},
	
	unarchiveUser: function (itemId) {
		var args = {
			"itemId": itemId,
			"archived": false
		}
		var srh = new ServiceResponseHandler(this, "unarchiveCallback", "unarchiveErrorCallback");
		AdminClient.postArchiveUser(args, srh);
	},
	
	unarchiveCallback: function (retVal) {
		var data = dojo.fromJson(retVal);
		if (data.errMessage) {
			this.displayErrorMessage(dojo.string.substitute(this.messages.errorUnarchiving, [data.errMessage]));
		}
		else {
			this.refresh();
		}
	},
	
	unarchiveErrorCallback: function (err) {
		this.displayErrorMessage(dojo.string.substitute(this.messages.errorUnarchiving, [err]));
	},
	
	createPagerDiv: function () {
		this.pagerDiv = document.createElement("div");
		dojo.toggleClass(this.pagerDiv, "pager", true);
		var prevSpan = document.createElement("span");
		var prevLink = document.createElement("a");
		prevLink.appendChild(document.createTextNode(this.messages.previous));
		prevSpan.appendChild(prevLink);
		var disabledPrev = document.createElement("span");
		disabledPrev.appendChild(document.createTextNode(this.messages.previous));
		dojo.toggleClass(disabledPrev, "disabledText", true);
		prevSpan.appendChild(disabledPrev);
		this.pagerDiv.appendChild(prevSpan);
		this.pagerDiv.prevSpan = prevSpan;
		this.pagerDiv.prevLink = prevLink;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var rangeSpan = document.createElement("span");
		dojo.toggleClass(rangeSpan, "bold", true);
		this.pagerDiv.appendChild(rangeSpan);
		this.pagerDiv.rangeSpan = rangeSpan;
		this.pagerDiv.appendChild(document.createTextNode(" " + this.messages.of + " "));
		var totalSpan = document.createElement("span");
		dojo.toggleClass(totalSpan, "bold", true);
		this.pagerDiv.appendChild(totalSpan);
		this.pagerDiv.totalSpan = totalSpan;
		this.pagerDiv.appendChild(document.createTextNode(" | "));
		var nextSpan = document.createElement("span");
		var nextLink = document.createElement("a");
		nextLink.appendChild(document.createTextNode(this.messages.next));
		nextSpan.appendChild(nextLink);
		var disabledNext = document.createElement("span");
		disabledNext.appendChild(document.createTextNode(this.messages.next));
		dojo.toggleClass(disabledNext, "disabledText", true);
		nextSpan.appendChild(disabledNext);
		this.pagerDiv.appendChild(nextSpan);
		this.pagerDiv.nextSpan = nextSpan;
		this.pagerDiv.nextLink = nextLink;
		this.tableControls.appendChild(this.pagerDiv);
		this.updatePagerDivLinks();
		this.connect(prevLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
		this.connect(nextLink,"onclick",function (e) {
			dojo.stopEvent(e);
			document.location.href = e.target.href;
		});
	},
	
	updatePagerDivLinks: function () {
		if (this.pagerDiv != null) {
			this.pagerDiv.prevLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage - 1);
			this.pagerDiv.nextLink.href = "#" + SEARCH_ACTION + "&" + this.latestSearchType + "=" + encodeURIComponent(this.latestSearchTerm) + "&" + "pageNum=" + (this.queryPage + 1);
		}
	}	
});

})();